<?php

namespace Drupal\role_expose\Tests;

use Drupal\simpletest\WebTestBase;


/**
 * Make sure content exists on logged in user profile page.
 *
 * @group Role Expose
 */
class RoleExposeRoleFormUsageTest extends WebTestBase {

  /**
   * The installation profile to use with this test.
   *
   * Don't need most of default core modules.
   *
   * @var string
   */
  // Let's see if the *default* profile 'testing' does the job.
  //protected $profile = 'minimal';

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = array('role_expose');

  /**
   * Implements setUp().
   */
  public function setUp() {
    parent::setUp();


  }

  /**
   * Make sure all user roles are listed in config page.
   */
  protected function testUserRoleFormUsage() {

    // More priviledged role:

    $user_admin = $this->createUser([], 'admin.user', TRUE);
    $this->drupalLogin($user_admin);

    // Set visibilities for test roles.
    $role_hidden = $this->createRole([], 'hidden', 'Role never visible');
    $this->drupalGet('admin/people/roles/manage/' . $role_hidden);
    $edit['role_expose'] = ROLE_EXPOSE_NEVER;
    $this->drupalPostForm(NULL, $edit, 'Save');

    $role_visible_when_has = $this->createRole([], 'visible_when_has', 'Role visble when has');
    $this->drupalGet('admin/people/roles/manage/' . $role_visible_when_has);
    $edit['role_expose'] = ROLE_EXPOSE_USER_WITH;
    $this->drupalPostForm(NULL, $edit, 'Save');

    $role_visible_when_not_has = $this->createRole([], 'visible_when_not_has', 'Role visible when not has');
    $this->drupalGet('admin/people/roles/manage/' . $role_visible_when_not_has);
    $edit['role_expose'] = ROLE_EXPOSE_USER_WITHOUT;
    $this->drupalPostForm(NULL, $edit, 'Save');

    $role_visible = $this->createRole([], 'visible', 'Role always visible');
    $this->drupalGet('admin/people/roles/manage/' . $role_visible);
    $edit['role_expose'] = ROLE_EXPOSE_ALWAYS;
    $this->drupalPostForm(NULL, $edit, 'Save');

    // Log the resulted Permissions -page (incl. roles and grants)
    $this->drupalGet('admin/people/permissions');

    // Grant some test roles to the basic account.
    $user_basic = $this->createUser([], 'basic.user');
    $this->drupalGet('user/' . $user_basic->id() . '/edit');
    $edit = [
      'roles[visible]' =>  TRUE,
      'roles[hidden]' =>  TRUE,
      'roles[visible_when_has]' =>  TRUE,
      'roles[visible_when_not_has]' =>  TRUE,
    ];
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertText(t('The changes have been saved.'), t('Roles saved'));

    // Check that we have NO roles visible in User profile page (no grants yet!)
    $this->drupalLogin($user_basic);
    $this->drupalGet('user');
    $this->assertNoText(t('Roles'), t('Role Expose -section is NOT visible (title)'));
    $this->assertNoText(t('Role always visible'), t('Role <em>Role always visible</em> is NOT printed in the UI'));
    $this->assertNoText(t('Role never visible'), t('Role <em>Role never visible</em> is NOT printed in the UI'));
    $this->assertNoText(t('Role visble when has'), t('Role <em>Role visble when has</em> is NOT printed in the UI'));
    $this->assertNoText(t('Role visble when not has'), t('Role <em>Role visble when not has</em> is NOT printed in the UI'));


    $this->drupalLogin($user_admin);

    // Grant some test roles to the priviledged account.
    $user_more_priviledged = $this->createUser([], 'priviledged.user');
    $role_see_own_roles = $this->createRole(['view own roles'], 'role_with_perms_1', 'Role with Role Expose perms');
    $this->drupalGet('user/' . $user_more_priviledged->id() . '/edit');
    $edit = [
      'roles[visible]' =>  TRUE,
      'roles[hidden]' =>  TRUE,
      'roles[visible_when_has]' =>  TRUE,
      'roles[visible_when_not_has]' =>  TRUE,
      'roles[role_with_perms_1]' => TRUE,
    ];
    $this->drupalPostForm(NULL, $edit, 'Save');
    $this->assertText(t('The changes have been saved.'), t('Roles saved'));

    // Check that we have roles in User profile page -  grants are now set!
    $this->drupalLogin($user_more_priviledged);
    $this->drupalGet('user');
    $this->assertText(t('Roles'), t('Role Expose -section visible (title)'));
    $this->assertText(t('Role always visible'), t('Role <em>Role always visible</em> is printed in the UI'));
    $this->assertNoText(t('Role never visible'), t('Role <em>Role never visible</em> is NOT printed in the UI'));
    $this->assertText(t('Role visble when has'), t('Role <em>Role visble when has</em> is printed in the UI'));
    $this->assertNoText(t('Role visble when not has'), t('Role <em>Role visble when not has</em> is NOT printed in the UI'));

    // Check that we have roles in User profile page -  grants are now set!
    $perms = [
      'access user profiles',
      // Without this perm user can't access other user profile page at all.
      'view roles of all users',
    ];
    $user_see_other_users_roles = $this->createUser($perms, 'see_all_user_roles.user');
    $this->drupalLogin($user_see_other_users_roles);
    // Go to see *other* user's profile, with 'view roles of all users' -perms.
    $this->drupalGet('user/' . $user_more_priviledged->id());
    $this->assertText(t('Roles'), t('Role Expose -section visible (title)'));
    $this->assertText(t('Role always visible'), t('Role <em>Role always visible</em> is printed in the UI'));
    $this->assertNoText(t('Role never visible'), t('Role <em>Role never visible</em> is NOT printed in the UI'));
    $this->assertText(t('Role visble when has'), t('Role <em>Role visble when has</em> is printed in the UI'));
    $this->assertNoText(t('Role visble when not has'), t('Role <em>Role visble when not has</em> is NOT printed in the UI'));


  }

}
